import java.awt.geom.Point2D;

interface Damageable
{
	public double getHealth ();
	
	public double getMaxHealth ();
	
	public void damage (double damage);
	
	public void heal (double health);
	
	public boolean isDamaged ();
	
	public void die ();
}

abstract class DamageableEntity extends Entity implements Damageable
{
	private double health;
	private double maxHealth;
	protected CircularCollisionArea collision;
	
	DamageableEntity (DPoint start, Layer layer, int radius, double health)
	{
		super(start, layer);
		
		this.maxHealth = health;
		this.health = health;
		
		collision = new CircularCollisionArea(this, radius);
	}
	
	public CollisionArea getCollisionArea ()
	{
		return collision;
	}
	
	public double getHealth ()
	{
		return health;
	}
	
	public double getMaxHealth ()
	{
		return maxHealth;
	}
	
	public void damage (double damage)
	{
		if (health <= 0)
		{
			return;
		}
		
		health -= damage;
		
		if (health <= 0)
		{
			health = 0;
			die();
		}
	}
	
	public void heal (double h)
	{
		health += h;
		
		if (health > maxHealth)
		{
			health = maxHealth;
		}
	}
	
	public boolean isDamaged ()
	{
		return (health < maxHealth);
	}
	
	public abstract void die ();
}
