import java.awt.*;

class Door extends Entity
{
	private boolean open = false;
	private String idCard;
	protected RectangularCollisionArea collision;
	
	Door (DPoint p, boolean open, String idCard, String orientation)
	{
		super(p, Layer.BELOW_PEOPLE);
		
		setSprite(Store.newSprite("door-" + orientation, false));
		
		if (open)
		{
			toggle();
		}
		
		this.idCard = idCard;
		
		if (orientation.equals("vertical"))
		{
			collision = new RectangularCollisionArea(this, -2, -16, 4, 32);
		}
		else
		{
			collision = new RectangularCollisionArea(this, -16, -2, 32, 4);
		}
	}
	
	public CollisionArea getCollisionArea ()
	{
		return collision;
	}
	
	public boolean isOpen ()
	{
		return open;
	}
	
	private void open ()
	{
		if (! open)
		{
			toggle();
		}
	}
	
	private void close ()
	{
		if (open)
		{
			toggle();
		}
	}
	
	private void toggle ()
	{
		open = (! open);
		sprite.nextFrame();
	}
	
	public void open (Player player)
	{
		if (idCard != null && player.hasCard(idCard))
		{
			open();
		}
	}
	
	public boolean solid ()
	{
		return (! open);
	}
	
	public void cycle (int time) {}
}

class VerticalDoor extends Door
{
	VerticalDoor (DPoint p, boolean open, String idCard)
	{
		super(p, open, idCard, "vertical");
	}
}

class HorizontalDoor extends Door
{
	HorizontalDoor (DPoint p, boolean open, String idCard)
	{
		super(p, open, idCard, "horizontal");
	}
}
