import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.util.*;

public abstract class Entity extends DPoint
{
	protected Sprite sprite;
	protected Layer layer;
	
	public Entity (DPoint start, Layer layer)
	{
		super(start);
		
		this.layer = layer;
	}
	
	public void setSprite (Sprite sprite)
	{
		this.sprite = sprite;
	}
	
	public Sprite getSprite ()
	{
		return sprite;
	}
	
	public abstract CollisionArea getCollisionArea ();
	
	public boolean solid ()
	{
		return false;
	}
	
	public void draw (Graphics g, Layer layer, int xOffset, int yOffset)
	{
		if (sprite == null || layer != this.layer)
		{
			return;
		}
		
		int relativeX = x() - xOffset;
		int relativeY = y() - yOffset;
		
		drawCentredImage(g, sprite.getImage(), relativeX, relativeY);
	}
	
	public boolean touching (Entity other)
	{
		return CollisionArea.touching(getCollisionArea(), other.getCollisionArea());
	}
	
	public abstract void cycle (int time);
	
	public static void drawCentredImage (Graphics g, Image image, int x, int y)
	{
		int drawX = x - image.getWidth(null) / 2;
		int drawY = y - image.getHeight(null) / 2;
		
		g.drawImage(image, drawX, drawY, null);
	}
	
	public static void drawRotatedImage (Graphics g, Image image, double radians, int x, int y)
	{
		if (! (g instanceof Graphics2D))
		{
			System.err.println("Oh dear.");
			return;
		}

		Graphics2D g2 = (Graphics2D) g;

		AffineTransform transform = new AffineTransform();
		transform.translate(x, y);
		transform.rotate(radians);
		transform.translate(-image.getWidth(null) / 2, -image.getHeight(null) / 2);

		if (image instanceof BufferedImage)
		{
			AffineTransformOp op = new AffineTransformOp(transform, AffineTransformOp.TYPE_BILINEAR);

			g2.drawImage((BufferedImage) image, op, 0, 0);
		}
		else
		{
			g2.drawImage(image, transform, null);
		}
	}
	
	public boolean equals (Object o)
	{
		return this == o;
	}
}
