import java.awt.*;

public class Explosion extends Entity
{
	private static final int TIME_INTERVAL = 30;
	private static final int RANGE_INCREASE = 2;
	private static final double DAMAGE_DECREASE = 0.05;
	
	private static final int TIME_LIMIT = 600;
	private static final int RANGE_LIMIT = 32;
	private static final double DAMAGE_LIMIT = 0.1;
	
	private int count = 0;
	
	private double damage = 0.5;
	
	protected CircularCollisionArea collision;
	
	public Explosion (DPoint spawn)
	{
		super(spawn, Layer.ABOVE_PEOPLE);
		
		Sprite sprite = Store.newSprite("explosion", true, 30);
		
		sprite.setLimit(1);
		
		setSprite(sprite);
		
		Store.playAudio("explosion");
		
		collision = new CircularCollisionArea(this, 8);
	}
	
	public CollisionArea getCollisionArea ()
	{
		return collision;
	}
	
	public void cycle (int elapsed)
	{
		do
		{
			if (count + elapsed >= TIME_INTERVAL)
			{
				int timeBefore = TIME_INTERVAL - count;
				
				cycle2(timeBefore);
				
				tick();
				
				count = 0;
				
				elapsed -= timeBefore;
			}
			else
			{
				cycle2(elapsed);
				
				count += elapsed;
				
				elapsed = 0;
			}
		}
		while (elapsed > 0);
	}
	
	private void cycle2 (int time)
	{
		if (sprite.isRunning())
		{
			for (Entity entity: GameArea.getLevel().entities)
			{
				if (entity instanceof Damageable)
				{
					if (touching(entity))
					{
						Damageable e = (Damageable) entity;
					
						e.damage(damage * time);
					}
				}
			}
		}
		else
		{
			GameArea.getLevel().removeEntity(this);
		}
	}
	
	private void tick ()
	{
		int radius = collision.getRadius();
		
		if (radius < RANGE_LIMIT)
		{
			collision.setRadius(radius + RANGE_INCREASE);
		}
		
		if (damage > DAMAGE_LIMIT)
		{
			damage -= DAMAGE_DECREASE;
		}
	}
	
	/* test code
	public static void main (String[] args)
	{
		Explosion e = new Explosion(new DPoint(0, 0));
		
		while (true)
		{
			e.cycle(29);
			try { Thread.sleep(100); } catch (Exception foo) {}
		}
	}
	*/
}
