import java.awt.*;

import java.awt.event.KeyEvent;

class GameArea extends Canvas implements Runnable, KeyTypedListener, InteractiveComponent
{
	private int width;
	private int height;
	private int halfWidth;
	private int halfHeight;
	
	private static GameArea instance;
	private static Level level;
	
	private static Level loadLevel;
	private static double loadX;
	private static double loadY;
	
	private static Thread thread;
	
	public static final DTimer timer = new DTimer();
	
	private static KeyPressedHandler keyPressedHandler;
	private static KeyTypedHandler keyTypedHandler;
	
	private Player player;
	
	GameArea ()
	{
		instance = this;
		
		keyPressedHandler = new KeyPressedHandler(new int[] {
			KeyEvent.VK_LEFT,
			KeyEvent.VK_RIGHT,
			KeyEvent.VK_UP,
			KeyEvent.VK_DOWN,
			KeyEvent.VK_SPACE,
			KeyEvent.VK_SHIFT,
		});
		
		keyTypedHandler = new KeyTypedHandler(this);
	}
	
	public void init ()
	{
		width = getWidth();
		height = getHeight();
		
		halfWidth = width / 2;
		halfHeight = height / 2;
		
		createBufferStrategy(2);
		
		level = Store.getLevel("0");
		
		player = Player.getInstance();
		
		keyTypedHandler.start();
		
		start();
	}
	
	public void start ()
	{
		if (thread != null)
		{
			System.err.println("Already started");
			return;
		}
		
		thread = new Thread(instance);
		
		thread.start();
	}
	
	public void run ()
	{
		keyPressedHandler.start();
		
		timer.start();
		
		long loopStart = timer.getTime();
		int elapsed;
		
		while (thread != null)
		{
			elapsed = (int) timer.timeSince(loopStart);
			
			if (elapsed < 1)
			{
				elapsed = 1;
			}
			else if (elapsed > 100)
			{
				elapsed = 100;
			}
			
			loopStart = timer.getTime();
			
			level.cycle(elapsed);
			
			if (loadLevel != null)
			{
				level.removeEntity(player);
				
				level = loadLevel;
				loadLevel = null;
				
				player.x = loadX;
				player.y = loadY;
				
				level.addEntity(player);
			}
			
			drawBuffer();
			
			Thread.yield();
		}
		
		timer.stop();
		
		keyPressedHandler.stop();
	}
	
	public void stop ()
	{
		thread = null;
	}
	
	public void end ()
	{
		stop();
		
		keyTypedHandler.stop();
	}
	
	public static boolean isRunning ()
	{
		return (thread != null);
	}
	
	public static boolean isPressed (int keyCode)
	{
		return keyPressedHandler.isPressed(keyCode);
	}
	
	public void keyTyped (char c)
	{
		char character = Character.toUpperCase(c);
		
		if (character == 'P')
		{
			if (isRunning())
			{
				stop();
			}
			else
			{
				start();
			}
			
			return;
		}
		
		if (! isRunning())
		{
			return;
		}
		
		switch (character)
		{
			case '0':
				player.setWeapon(0);
				break;
			case '1':
				player.setWeapon(1);
				break;
			case '2':
				player.setWeapon(2);
				break;
			case '3':
				player.setWeapon(3);
				break;
		}
	}
	
	public static void load (String level, double x, double y)
	{
		loadLevel = Store.getLevel(level);
		
		loadX = x;
		loadY = y;
	}
	
	public void drawBuffer ()
	{
		if (! isDisplayable())
		{
			return;
		}
		
		int x = player.x() - halfWidth;
		int y = player.y() - halfHeight;
		
		Graphics g = getBufferStrategy().getDrawGraphics();
		
		g.setColor(Color.BLACK);
		g.fillRect(0, 0, width, height);
		
		level.draw(g, x, y);
		
		double health = player.getHealth() / player.getMaxHealth();
		
		if (health > 0)
		{
			g.setColor(Color.DARK_GRAY);
			
			g.fillRect(20, 20, 20, height - 40);
			
			g.fillRect(50, 20, 32, 32);
			
			g.setColor(new Color((float) (1 - health), (float) health, 0));
			
			int healthH = (int) (health * (height - 44));
			
			g.fillRect(22, height - 22 - healthH, 16, healthH);
			
			g.setColor(Color.WHITE);
			
			g.fillRect(52, 22, 28, 28);
			
			Weapon w = player.getWeapon();
			
			if (w instanceof Pistol)
			{
				Entity.drawCentredImage(g, Store.getImage("pickup-pistol"), 66, 36);
			}
			else if (w instanceof MachineGun)
			{
				Entity.drawCentredImage(g, Store.getImage("pickup-machinegun"), 66, 36);
			}
			/*else if (w instanceof Bazooka)
			{
				Entity.drawCentredImage(g, Store.getImage("pickup-bazooka"), 66, 36);
			}*/
		}/*
		else
		{
			Entity.drawCentredImage(g, Store.getImage("game-over"), width / 2, height / 2);
		}*/
		
		display();
	}
	
	public void display ()
	{
		try
		{
			getBufferStrategy().show();
		}
		catch (Exception e) {}
	}
	
	public static GameArea getInstance ()
	{
		return instance;
	}
	
	public static Level getLevel ()
	{
		return level;
	}
	
	public void paint (Graphics g)
	{
		if (! isRunning())
		{
			display();
		}
	}
}
