import java.awt.*;
import java.awt.event.*;

abstract class ImageScreen extends Canvas implements MouseListener, Runnable, InteractiveComponent
{
	private Image image;
	
	private boolean active;
	
	private int timeout;
	
	ImageScreen (Image image)
	{
		this(image, 0);
	}
	
	ImageScreen (Image image, int timeout)
	{
		this.image = image;
		this.timeout = timeout;
		
		active = true;
		
		addMouseListener(this);
		
		if (timeout > 0)
		{
			Thread thread = new Thread(this);
			
			thread.start();
		}
	}
	
	public abstract void action ();
	
	public void run ()
	{
		try
		{
			Thread.sleep(timeout);
		}
		catch (InterruptedException e)
		{
			return;
		}
		
		if (! active)
		{
			return;
		}
		
		action();
	}
	
	public void paint (Graphics g)
	{
		int x = (getWidth() - image.getWidth(null)) / 2;
		int y = (getHeight() - image.getHeight(null)) / 2;
		g.drawImage(image, x, y, null);
	}
	
	public void init () {}
	public void start () {}
	public void stop () {}
	public void end ()
	{
		active = false;
	}
	
	public void mouseClicked (MouseEvent e)
	{
		action();
	}
	
	public void mouseEntered (MouseEvent e) {}
	
	public void mouseExited (MouseEvent e) {}
	
	public void mousePressed (MouseEvent e) {}
	
	public void mouseReleased (MouseEvent e) {}
}
