import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class KeyPressedHandler implements KeyEventDispatcher
{
	private final int[] keyCodes;
	
	private final Map <Integer, KeyStatus> lookup = new HashMap <Integer, KeyStatus> ();
	
	public KeyPressedHandler (int[] keyCodes)
	{
		this.keyCodes = keyCodes;
	}
	
	public void start ()
	{
		for (int i = 0; i < keyCodes.length; i++)
		{
			lookup.put(keyCodes[i], new KeyStatus());
		}
		
		KeyboardFocusManager.getCurrentKeyboardFocusManager().addKeyEventDispatcher(this);
	}
	
	public void stop ()
	{
		KeyboardFocusManager.getCurrentKeyboardFocusManager().removeKeyEventDispatcher(this);
	}
	
	public boolean dispatchKeyEvent (KeyEvent e)
	{
		int keyCode = e.getKeyCode();
		
		KeyStatus key = lookup.get(keyCode);
		
		if (key != null)
		{
			if (e.getID() == KeyEvent.KEY_PRESSED)
			{
				key.pressed();
			}
			else if (e.getID() == KeyEvent.KEY_RELEASED)
			{
				key.released();
			}
		}
		
		return false;
	}
	
	public boolean isPressed (int keyCode)
	{
		KeyStatus key = lookup.get(keyCode);
		
		if (key != null)
		{
			return key.isPressed();
		}
		else
		{
			return false;
		}
	}
}

class KeyStatus
{
	boolean pressed = false;
	boolean pressedLastTime = false;
	boolean typed = false;
	
	public void pressed ()
	{
		pressed = true;
	}
	
	public void released ()
	{
		if (pressed && ! pressedLastTime)
		{
			typed = true;
		}
		
		pressed = false;
	}
	
	public boolean isPressed ()
	{
		boolean ret = pressed || typed;
	
		pressedLastTime = pressed;
		
		typed = false;
		
		return ret;
	}
}
