import java.util.*;
import java.io.*;
import java.net.*;

public class LineReader extends BufferedReader implements Iterable<String>
{
	public LineReader (File file) throws FileNotFoundException
	{
		super(new FileReader(file));
	}
	
	public LineReader (String string) throws IOException
	{
		this(Store.getURL(string));
	}
	
	public LineReader (URL url) throws IOException
	{
		this(url.openStream());
	}
	
	public LineReader (InputStream stream)
	{
		super(new InputStreamReader(stream));
	}
	
	public Iterator<String> iterator ()
	{
		return new LineReaderIterator(this);
	}
}

class LineReaderIterator implements Iterator<String>
{
	private BufferedReader reader;
	private String next;
	
	public LineReaderIterator (BufferedReader r)
	{
		reader = r;
		
		try
		{
			next = reader.readLine();
		}
		catch (IOException e)
		{
			next = null;
		}
	}
	
	public boolean hasNext ()
	{
		return (next != null);
	}
	
	public String next ()
	{
		String current = next;
		
		try
		{
			next = reader.readLine();
		}
		catch (IOException e)
		{
			next = null;
		}
		
		return current;
	}
	
	public void remove () throws UnsupportedOperationException
	{
		throw new UnsupportedOperationException();
	}
}
