import java.awt.geom.Point2D;

abstract class MovingEntity extends RotatingEntity
{
	public MovingEntity (DPoint start, Layer layer, int radius)
	{
		super(start, layer, radius);
	}
	
	public MovingEntity (DPoint start, Layer layer, int radius, double direction)
	{
		super(start, layer, radius, direction);
	}
	
	public boolean move (double distance)
	{
		DPoint there = new DPoint(this);
		
		there.translate(distance, 0, direction.getValue());
		
		if (valid(there))
		{
			setLocation(there);
			collision.setPosition(there);
			
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public boolean valid (double x, double y)
	{
		DPoint p = new DPoint(x, y);
		
		return valid(p);
	}
	
	public boolean valid (DPoint p)
	{
		collision.setPosition(p);
		
		Tile[] tiles = GameArea.getLevel().getTiles(collision);
		
		for (int i = 0; i < tiles.length; i++)
		{
			Tile tile = tiles[i];
			
			if (this instanceof Projectile)
			{
				if (tile.type.equals("wall") && collision.touching(tile))
				{
					collision.setPosition(this);
					return false;
				}
			}
			else
			{
				if (! tile.type.equals("floor") && collision.touching(tile))
				{
					collision.setPosition(this);
					return false;
				}
			}
		}
		
		for (Entity entity: GameArea.getLevel().entities)
		{
			if (this != entity && touching(entity))
			{
				if (collision(entity))
				{
					collision.setPosition(this);
					return false;
				}
			}
		}
		
		collision.setPosition(this);
		
		return true;
	}
	
	public boolean wouldTouch (DPoint p, Entity other)
	{
		collision.setPosition(p);
		boolean ret = touching(other);
		collision.setPosition(this);
		
		return ret;
	}
	
	public boolean collision (Entity entity)
	{
		return (entity.solid());
	}
}
