import java.util.*;

import java.awt.event.KeyEvent;

public class Player extends ArmedHuman
{
	private static Player instance;
	
	private LinkedList <String> cards;
	
	public static Player getInstance ()
	{
		return instance;
	}
	
	Player (DPoint spawn)
	{
		super(spawn, "player", 100);
		
		setWeapons(new Weapon[] {
			new Unarmed(this),
			new Pistol(this),
			new MachineGun(this),
			new Bazooka(this),
		});
		
		setDirection(Direction.NORTH);
		
		instance = this;
		
		cards = new LinkedList <String> ();
	}
	
	public void die ()
	{
		DeadBody body = new DeadPlayer(this);
		
		GameArea.getLevel().addEntity(body);
		
		GameArea.getLevel().removeEntity(this);
	}
	
	public boolean pickup (Pickup pickup)
	{
		if (pickup instanceof CardPickup)
		{
			CardPickup card = (CardPickup) pickup;
			
			cards.add(card.id);
			
			return true;
		}
		else if (pickup instanceof HealthPickup)
		{
			if (isDamaged())
			{
				heal(10);
				return true;
			}
		}
		else if (pickup instanceof PistolAmmo)
		{
			return true;
		}
		
		return false;
	}
	
	public boolean hasCard (String id)
	{
		for (String cardID: cards)
		{
			if (id.equals(cardID))
			{
				return true;
			}
		}
		
		return false;
	}
	
	public boolean collision (Entity e)
	{
		if (e instanceof Door)
		{
			Door door = (Door) e;
			
			door.open(this);
		}
		
		return super.collision(e);
	}
	
	public void cycle (int time)
	{
		boolean left = GameArea.isPressed(KeyEvent.VK_LEFT);
		boolean right = GameArea.isPressed(KeyEvent.VK_RIGHT);
		boolean up = GameArea.isPressed(KeyEvent.VK_UP);
		boolean down = GameArea.isPressed(KeyEvent.VK_DOWN);
		boolean space = GameArea.isPressed(KeyEvent.VK_SPACE);
		boolean shift = GameArea.isPressed(KeyEvent.VK_SHIFT);
		
		if (left && ! right)
		{
			turnLeft(time);
		}
		else if (! left && right)
		{
			turnRight(time);
		}
		
		if (up && ! down)
		{
			if (shift && ! (weapon instanceof Bazooka))
			{
				move(RUN, time);
			}
			else
			{
				move(WALK, time);
			}
		}
		else if (! up && down)
		{
			move(BACK, time);
		}
		else
		{
			move(STILL, time);
		}
		
		if (space)
		{
			weapon.fire();
		}
	}
}
