import java.util.*;

class Regulator extends DamageableEntity
{
	private long startedFixing = 0;
	private long lastFixed = 0;
	
	private boolean dead = false;
	
	private static List<Regulator> list;
	
	Regulator (DPoint spawn)
	{
		super(spawn, Layer.PEOPLE, 16, 60);
		
		setSprite(Store.newSprite("regulator"));
		
		list.add(this);
	}
	
	public static void resetList ()
	{
		list = new LinkedList<Regulator>();
	}
	
	public static boolean allDead ()
	{
		for (Regulator r: list)
		{
			if (! r.dead)
			{
				return false;
			}
		}
		
		return true;
	}
	
	public void heal (double health)
	{
		if (dead)
		{
			lastFixed = GameArea.timer.getTime();
			
			if (startedFixing == 0)
			{
				startedFixing = lastFixed;
			}
			
			if (GameArea.timer.timeSince(startedFixing) < 5000)
			{
				return;
			}
			
			dead = false;
			
			setSprite(Store.newSprite("regulator"));
		}
		
		super.heal(health);
	}
	
	public void die ()
	{
		dead = true;
		
		GameArea.getLevel().addEntity(new Explosion(this));
		
		setSprite(Store.newSprite("regulator-broken"));
		
		if (allDead())
		{
			Main.setScreen(new WinScreen());
		}
	}
	
	public void cycle (int time)
	{
		if ((startedFixing != 0) && (GameArea.timer.timeSince(lastFixed) > 500))
		{
			startedFixing = 0;
		}
	}
	
	public boolean solid ()
	{
		return true;
	}
}
