import java.awt.*;
import java.awt.geom.Point2D;

abstract class RotatingEntity extends Entity
{
	protected Direction direction;
	protected CircularCollisionArea collision;
	
	public RotatingEntity (DPoint start, Layer layer, int radius)
	{
		super(start, layer);
		
		this.direction = new Direction();
		
		collision = new CircularCollisionArea(this, radius);
	}
	
	public RotatingEntity (DPoint start, Layer layer, int radius, double direction)
	{
		super(start, layer);
		
		this.direction = new Direction(direction);
		
		collision = new CircularCollisionArea(this, radius);
	}
	
	public CollisionArea getCollisionArea ()
	{
		return collision;
	}
	
	public void translate (double forwards, double sideways)
	{
		translate(forwards, sideways, direction.getValue());
	}
	
	public void setDirection (double d)
	{
		direction.setValue(d);
	}
	
	public double getDirection ()
	{
		return direction.getValue();
	}
	
	public void turnLeft (int time)
	{
		direction.turnLeft(time);
	}
	
	public void turnRight (int time)
	{
		direction.turnRight(time);
	}
	
	public double turnTowards (DPoint there, double speed)
	{
		return turnTowards(direction(there), speed);
	}
	
	public double turnTowards (double there, double speed)
	{
		double diff = direction.relative(there);
		
		if (diff < -speed)
		{
			direction.turnRight(speed);
			diff += speed;
		}
		else if (diff > speed)
		{
			direction.turnLeft(speed);
			diff -= speed;
		}
		else
		{
			direction.turnLeft(diff);
			diff = 0;
		}
		
		return diff;
	}
	
	public double relativeDirectionTo (DPoint there)
	{
		return relativeDirectionTo(direction(there));
	}
	
	public double relativeDirectionTo (double d)
	{
		return direction.relative(d);
	}
	
	public void draw (Graphics g, Layer layer, int xOffset, int yOffset)
	{
		if (sprite == null || layer != this.layer)
		{
			return;
		}
		
		int relativeX = x() - xOffset;
		int relativeY = y() - yOffset;
		
		double radians = direction.toRadians();
		
		drawRotatedImage(g, sprite.getImage(), radians, relativeX, relativeY);
	}
}
