import java.awt.*;

public class Sprite
{
	private Image[] frames;
	
	private int currentFrame;
	private long lastChange = 0;
	
	private int time;
	private boolean running;
	
	private int limit = 0;
	private int count = 0;
	
	public Sprite (Image image)
	{
		frames = new Image[] { image };
		
		running = false;
		
		time = 10000;
	}
	
	public Sprite (Image[] frames)
	{
		this(frames, true, 100);
	}
	
	public Sprite (Image[] frames, boolean start)
	{
		this(frames, start, 100);
	}
	
	public Sprite (Image[] frames, int time)
	{
		this(frames, true, time);
	}
	
	public Sprite (Image[] frames, boolean start, int time)
	{
		this.frames = frames;
		this.running = start;
		this.time = time;
	}
	
	public void setFrameTime (int time)
	{
		this.time = time;
	}
	
	public void setLimit (int limit)
	{
		this.limit = limit;
	}
	
	public void start ()
	{
		if (! running)
		{
			forceStart();
		}
	}
	
	private void forceStart ()
	{
		count = 0;
		running = true;
		lastChange = GameArea.timer.getTime();
	}
	
	public void stop ()
	{
		running = false;
		currentFrame = 0;
	}
	
	public void nextFrame ()
	{
		currentFrame++;
		
		if (currentFrame == frames.length)
		{
			currentFrame = 0;
			count++;
			
			if (limit > 0 && count >= limit)
			{
				stop();
				
				//currentFrame = frames.length - 1;
			}
		}
	}
	
	private void checkTime ()
	{
		if (lastChange == 0)
		{
			forceStart();
		}
		else
		{
			while (GameArea.timer.timeSince(lastChange) > time)
			{
				lastChange += time;
				
				nextFrame();
			}
		}
	}
	
	public Image getImage ()
	{
		if (running)
		{
			checkTime();
		}
		
		return frames[currentFrame];
	}
	
	public boolean isRunning ()
	{
		return running;
	}
	
	public Image[] getFrames ()
	{
		return frames;
	}
}
