import java.applet.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import javax.imageio.*;
import java.util.*;
import java.net.*;
import java.io.*;
import javax.sound.sampled.*;

public class Store
{
	public static final Map <String, AudioClip> sounds = new HashMap <String, AudioClip> ();
	public static final Map <String, Image> images = new HashMap <String, Image> ();
	public static final Map <String, Level> levels = new HashMap <String, Level> ();
	public static final Map <String, TileData> tiles = new HashMap <String, TileData> ();
	public static final Map <String, Image[]> sprites = new HashMap <String, Image[]> ();
	
	public static void load () throws IOException
	{
		LineReader lineReader = new LineReader("images/list.txt");
		
		for (String file: lineReader)
		{
			loadImage(file);
		}
		
		lineReader = new LineReader("sprites/list.txt");
		
		for (String file: lineReader)
		{
			loadSprite(file);
		}
		
		lineReader = new LineReader("tiles/list.txt");
		
		for (String file: lineReader)
		{
			loadTiles(file);
		}
		
		lineReader = new LineReader("audio/list.txt");
		
		for (String file: lineReader)
		{
			loadAudio(file);
		}
	}
	
	public static void loadImage (String file) throws IOException
	{
		String[] parts = file.split("\\.");
		
		if (parts.length != 2)
		{
			return;
		}
		
		if (! parts[1].toLowerCase().equals("gif"))
		{
			return;
		}
		
		String name = parts[0];
		
		Image image = ImageIO.read(getURL("images/" + file));
		
		images.put(name, image);
	}
	
	public static void loadSprite (String file) throws IOException
	{
		String[] parts = file.split("\\.");
		
		if (parts.length != 2)
		{
			return;
		}
		
		if (! parts[1].toLowerCase().equals("gif"))
		{
			return;
		}
		
		String name = parts[0];
		
		Image image = ImageIO.read(getURL("sprites/" + file));
		
		sprites.put(name, parse(image));
	}
	
	public static void loadTiles (String file) throws IOException
	{
		String[] parts = file.split("\\.");
		
		if (parts.length != 3)
		{
			return;
		}
		
		if (! parts[2].toLowerCase().equals("gif"))
		{
			return;
		}
		
		String key = parts[0];
		
		String name = parts[1];
		
		Image[] images = parse(ImageIO.read(getURL("tiles/" + file)));
		
		for (int i = 0; i < images.length; i++)
		{
			tiles.put(key + i, new TileData(name, images[i]));
		}
	}
	
	public static void loadAudio (String file)
	{
		String[] parts = file.split("\\.");
		
		if (parts.length != 2)
		{
			return;
		}
		
		if (! parts[1].toLowerCase().equals("wav"))
		{
			return;
		}
		
		String name = parts[0];
		
		AudioClip audio = Applet.newAudioClip(getURL("audio/" + file));
		
		sounds.put(name, audio);
		
		audio.play();
		audio.stop();
	}
	
	public static Image[] parse (Image image)
	{
		BufferedImage bimage = toBufferedImage(image);
		
		int size = bimage.getHeight(null);
		
		int count = bimage.getWidth(null) / size;
		
		Image[] array = new Image[count];
		
		for (int i = 0; i < count; i++)
		{
			array[i] = bimage.getSubimage(i * size, 0, size, size);
		}
		
		return array;
	}
	
	public static URL getURL (String file)
	{
		return Main.getInstance().getClass().getResource(file);
	}
	
	public static LinkedList <String> readFile (String file) throws IOException
	{
		LinkedList <String> lines = new LinkedList <String> ();
		
		LineReader lineReader = new LineReader(file);
		
		for (String s: lineReader)
		{
			lines.add(s);
		}
		
		return lines;
	}
	
	public static void addLevel (String name)
	{
		LinkedList <String> lines = null;
		
		try
		{
			lines = readFile("levels/" + name + ".txt");
		}
		catch (IOException e)
		{
			System.err.println("Error: Level " + name + " could not be loaded.");
			System.exit(1);
		}
		
		String[] objects = lines.removeFirst().split(" ");
		
		int ySize = lines.size();
		int xSize = lines.getFirst().split(" ").length;
		
		Tile[][] grid = new Tile[ySize][xSize];
		
		int y = 0;
		
		for (String line: lines)
		{
			String[] tiles = line.split(" ");
			
			for (int x = 0; x < xSize; x++)
			{
				grid[y][x] = newTile(tiles[x], x, y);
				
				if (grid[y][x] == null)
				{
					System.err.println("Tile " + tiles[x] + "does not exist");
				}
			}
			
			y++;
		}
		
		levels.put(name, new Level(grid, objects));
	}
	
	public static void loadLevels ()
	{
		LineReader lineReader = null;
		
		try
		{
			lineReader = new LineReader("levels/list.txt");
		}
		catch (IOException e)
		{
			System.err.println("Error: Level list could not be loaded.");
			System.exit(1);
		}
		
		for (String s: lineReader)
		{
			s = s.trim();
			
			if (! s.equals(""))
			{
				addLevel(s);
			}
		}
	}
	
	public static AudioClip getAudio (String name)
	{
		return sounds.get(name);
	}
	
	public static void playAudio (String name)
	{
		AudioClip audio = getAudio(name);
		
		if (audio != null)
		{
			audio.play();
		}
	}
	
	public static void stopAudio (String name)
	{
		AudioClip audio = getAudio(name);
		
		if (audio != null)
		{
			audio.stop();
		}
	}
	
	public static Image getImage (String name)
	{
		return images.get(name);
	}
	
	public static Level getLevel (String name)
	{
		return levels.get(name);
	}
	
	public static TileData getTileData (String name)
	{
		return tiles.get(name);
	}
	
	public static Tile newTile (String name, int x, int y)
	{
		return new Tile(tiles.get(name), x, y);
	}
	
	public static Image[] getSprite (String name)
	{
		return sprites.get(name);
	}
	
	public static Sprite newSprite (String name)
	{
		return new Sprite(getSprite(name));
	}
	
	public static Sprite newSprite (String name, boolean start)
	{
		return new Sprite(getSprite(name), start);
	}
	
	public static Sprite newSprite (String name, int time)
	{
		return new Sprite(getSprite(name), time);
	}
	
	public static Sprite newSprite (String name, boolean start, int time)
	{
		return new Sprite(getSprite(name), start, time);
	}
	
	public static Entity newEntity (String[] params, DPoint position)
	{
		String type = params[0];
		
		Entity entity = null;
		
		if (type.equals("Player"))
		{
			entity = new Player(position);
		}
		else if (type.equals("Scientist"))
		{
			entity = new Scientist(position);
		}
		else if (type.equals("Mechanic"))
		{
			entity = new Mechanic(position);
		}
		else if (type.equals("Guard"))
		{
			entity = new Guard(position);
		}
		else if (type.equals("Stairs"))
		{
			String updown = params[1];
			String level = params[2];
			int spawnX = Integer.parseInt(params[3]);
			int spawnY = Integer.parseInt(params[4]);
			
			entity = new Stairs(position, level, spawnX, spawnY, updown);
		}
		else if (type.equals("DoorH"))
		{
			boolean open = (params[1].equals("open"));
			
			String idCard = null;
			
			if (params.length == 3)
			{
				idCard = params[2];
			}
			
			entity = new HorizontalDoor(position, open, idCard);
		}
		else if (type.equals("DoorV"))
		{
			boolean open = (params[1].equals("open"));
			
			String idCard = null;
			
			if (params.length == 3)
			{
				idCard = params[2];
			}
			
			entity = new VerticalDoor(position, open, idCard);
		}
		else if (type.equals("PCard"))
		{
			entity = new CardPickup(position, params[1]);
		}
		else if (type.equals("PHealth"))
		{
			entity = new HealthPickup(position);
		}
		else if (type.equals("PistolAmmo"))
		{
			entity = new PistolAmmo(position);
		}
		else if (type.equals("Regulator"))
		{
			entity = new Regulator(position);
		}
		else if (type.equals("Trigger"))
		{
			entity = new Trigger(position);
		}
		
		if (entity instanceof Human)
		{
			for (int i = 1; i < params.length; i++)
			{
				if (params[i].charAt(0) == '!')
				{
					String[] object = params[i].substring(1).split(";");
					
					((Human) entity).setCarrying(object);
				}
			}
		}
		
		return entity;
	}
	
	// Following three methods from http://javaalmanac.com/
	
	// This method returns a buffered image with the contents of an image
    public static BufferedImage toBufferedImage(Image image) {
        if (image instanceof BufferedImage) {
            return (BufferedImage)image;
        }
    
        // This code ensures that all the pixels in the image are loaded
        image = new ImageIcon(image).getImage();
    
        // Determine if the image has transparent pixels; for this method's
        // implementation, see e661 Determining If an Image Has Transparent Pixels
        boolean hasAlpha = hasAlpha(image);
    
        BufferedImage bimage = newBufferedImage(image.getWidth(null), image.getHeight(null), hasAlpha);
    
        // Copy image to buffered image
        Graphics g = bimage.createGraphics();
    
        // Paint the image onto the buffered image
        g.drawImage(image, 0, 0, null);
        g.dispose();
    
        return bimage;
    }
    
    public static BufferedImage newBufferedImage (int w, int h, boolean transparent)
    {
    	// Create a buffered image with a format that's compatible with the screen
        BufferedImage image = null;
        
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        try {
            // Determine the type of transparency of the new buffered image
            int transparency = (transparent ? Transparency.BITMASK : Transparency.OPAQUE);
    
            // Create the buffered image
            GraphicsDevice gs = ge.getDefaultScreenDevice();
            GraphicsConfiguration gc = gs.getDefaultConfiguration();
            image = gc.createCompatibleImage(w, h, transparency);
        } catch (HeadlessException e) {
            // The system does not have a screen
        }
    
        if (image == null) {
            // Create a buffered image using the default color model
            int type = (transparent ? BufferedImage.TYPE_INT_ARGB : BufferedImage.TYPE_INT_RGB);
            
            image = new BufferedImage(w, h, type);
        }
        
        return image;
    }
    
    // This method returns true if the specified image has transparent pixels
    public static boolean hasAlpha(Image image) {
        // If buffered image, the color model is readily available
        if (image instanceof BufferedImage) {
            BufferedImage bimage = (BufferedImage)image;
            return bimage.getColorModel().hasAlpha();
        }
    
        // Use a pixel grabber to retrieve the image's color model;
        // grabbing a single pixel is usually sufficient
         PixelGrabber pg = new PixelGrabber(image, 0, 0, 1, 1, false);
        try {
            pg.grabPixels();
        } catch (InterruptedException e) {
        }
    
        // Get the image's color model
        ColorModel cm = pg.getColorModel();
        return cm.hasAlpha();
    }
}
