

abstract class Weapon
{
	protected Human owner;
	
	public final Sprite sprite;
	public final int waitTime;
	public final boolean automaticFire;
	
	private long lastFired = 0;
	
	Weapon (Human owner, String type, int wait)
	{
		this(owner, type, wait, false);
	}
	
	Weapon (Human owner, String type, int wait, boolean auto)
	{
		this.owner = owner;
		
		waitTime = wait;
		
		automaticFire = auto;
		
		sprite = Store.newSprite(owner.type + "-" + type, false);
	}
	
	public void fire ()
	{
		if (GameArea.timer.timeSince(lastFired) > waitTime)
		{
			lastFired = GameArea.timer.getTime();
			
			fireAction();
		}
	}
	
	abstract protected void fireAction ();
}

class Unarmed extends Weapon
{
	Unarmed (Human owner)
	{
		super(owner, "unarmed", 0);
	}
	
	protected void fireAction () {}
}

class Pistol extends Weapon
{
	private static final double forwards = 16;
	private static final double sideways = 10;
	
	Pistol (Human owner)
	{
		super(owner, "pistol", 500);
	}
	
	protected void fireAction ()
	{
		Store.playAudio("pistol");
		
		Bullet bullet = new Bullet(owner.getLocation(), owner.getDirection(), 8);
		
		bullet.translate(forwards, sideways);
		
		GameArea.getLevel().addEntity(bullet);
	}
}

class MachineGun extends Weapon
{
	private static int i;
	
	private static final double forwards = 17;
	private static final double sideways = 6.5;
	
	MachineGun (Human owner)
	{
		super(owner, "machinegun", 25, true);
	}
	
	protected void fireAction ()
	{
		Store.playAudio("machinegun" + (i++ % 3));
		
		Bullet bullet = new Bullet(owner.getLocation(), owner.getDirection(), 2);
		
		bullet.translate(forwards, sideways);
		
		bullet.move(Math.random() * 20);
		
		GameArea.getLevel().addEntity(bullet);
	}
}

class Bazooka extends Weapon
{
	private static final double forwards = 20;
	private static final double sideways = 7;
	
	Bazooka (Human owner)
	{
		super(owner, "bazooka", 2000);
	}
	
	protected void fireAction ()
	{
		Store.playAudio("rocket-fire");
		
		Rocket rocket = new Rocket(owner.getLocation(), owner.getDirection());
		
		rocket.translate(forwards, sideways);
		
		GameArea.getLevel().addEntity(rocket);
	}
}
